# Generated by Django 4.2.10 on 2025-05-09 05:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0063_rpmpublication_layout_rpmrepository_layout'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rpmrepository',
            name='original_checksum_types',
        ),
        migrations.AlterField(
            model_name='rpmpublication',
            name='metadata_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')], null=True),
        ),
        migrations.AlterField(
            model_name='rpmpublication',
            name='package_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')], null=True),
        ),
    ]
