# Generated by Django 4.2.10 on 2025-07-01 20:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0064_remove_rpmrepository_original_checksum_types_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='package',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('upload_rpm_packages', 'Can upload RPM packages using synchronous API.')]},
        ),
    ]
