# Generated by Django 3.2.13 on 2022-05-17 20:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0042_alter_repometadatafile_data_type'),
    ]

    operations = [
        migrations.AlterField(
            model_name='addon',
            name='addon_id',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='addon',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='addon',
            name='packages',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='addon',
            name='type',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='addon',
            name='uid',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='checksum',
            name='checksum',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='checksum',
            name='path',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='arch',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='base_product_name',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='base_product_short',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='base_product_version',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='digest',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='header_version',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='instimage',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='mainimage',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='release_name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='release_short',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='release_version',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='image',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='image',
            name='path',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='image',
            name='platforms',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='arch',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='context',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='stream',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='version',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemddefaults',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='modulemddefaults',
            name='module',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='modulemddefaults',
            name='stream',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='package',
            name='checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')]),
        ),
        migrations.AlterField(
            model_name='package',
            name='pkgId',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='packagecategory',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='packagecategory',
            name='id',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packagecategory',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packageenvironment',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='packageenvironment',
            name='id',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packageenvironment',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packagegroup',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='packagegroup',
            name='id',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packagegroup',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='packagelangpacks',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='repometadatafile',
            name='checksum',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='repometadatafile',
            name='checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')]),
        ),
        migrations.AlterField(
            model_name='rpmpublication',
            name='metadata_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')]),
        ),
        migrations.AlterField(
            model_name='rpmpublication',
            name='package_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')]),
        ),
        migrations.AlterField(
            model_name='rpmremote',
            name='sles_auth_token',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='rpmrepository',
            name='metadata_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')], null=True),
        ),
        migrations.AlterField(
            model_name='rpmrepository',
            name='package_checksum_type',
            field=models.TextField(choices=[('unknown', 'unknown'), ('md5', 'md5'), ('sha1', 'sha1'), ('sha1', 'sha1'), ('sha224', 'sha224'), ('sha256', 'sha256'), ('sha384', 'sha384'), ('sha512', 'sha512')], null=True),
        ),
        migrations.AlterField(
            model_name='ulnremote',
            name='uln_server_base_url',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='updaterecord',
            name='digest',
            field=models.TextField(unique=True),
        ),
        migrations.AlterField(
            model_name='updaterecord',
            name='id',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='debug_packages',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='debug_repository',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='identity',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='name',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='variant',
            name='packages',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='variant',
            name='source_packages',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='source_repository',
            field=models.TextField(null=True),
        ),
        migrations.AlterField(
            model_name='variant',
            name='type',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='variant',
            name='uid',
            field=models.TextField(),
        ),
        migrations.AlterField(
            model_name='variant',
            name='variant_id',
            field=models.TextField(),
        ),
    ]
