# Generated by Django 4.2.10 on 2025-02-12 19:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0062_rpmpackagesigningservice_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='rpmpublication',
            name='layout',
            field=models.TextField(choices=[('nested_alphabetically', 'nested_alphabetically'), ('flat', 'flat')], null=True),
        ),
        migrations.AddField(
            model_name='rpmrepository',
            name='layout',
            field=models.TextField(choices=[('nested_alphabetically', 'nested_alphabetically'), ('flat', 'flat')], null=True),
        ),
    ]
