# Generated by Django 4.2.7 on 2023-12-12 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0058_alter_addon_repository_alter_variant_repository'),
    ]

    operations = [
        migrations.AddField(
            model_name='rpmpublication',
            name='compression_type',
            field=models.TextField(choices=[('zstd', 'zstd'), ('gz', 'gz')], null=True),
        ),
        migrations.AddField(
            model_name='rpmrepository',
            name='compression_type',
            field=models.TextField(choices=[('zstd', 'zstd'), ('gz', 'gz')], null=True),
        ),
    ]
