# Generated by Django 4.2.1 on 2023-05-12 13:24

from django.db import migrations, models
import django.db.models.deletion
import pulpcore.app.models.base


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0106_alter_artifactdistribution_distribution_ptr_and_more'),
        ('rpm', '0049_profiles_fix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='addon',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='checksum',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='distributiontree',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='image',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='modulemd',
            name='packages',
            field=models.ManyToManyField(to='rpm.package'),
        ),
        migrations.AlterField(
            model_name='modulemddefaults',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='modulemdobsolete',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='package',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='packagecategory',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='packageenvironment',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='packagegroup',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='packagelangpacks',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='repometadatafile',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='rpmalternatecontentsource',
            name='alternatecontentsource_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.alternatecontentsource'),
        ),
        migrations.AlterField(
            model_name='rpmdistribution',
            name='distribution_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.distribution'),
        ),
        migrations.AlterField(
            model_name='rpmpublication',
            name='publication_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.publication'),
        ),
        migrations.AlterField(
            model_name='rpmremote',
            name='remote_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.remote'),
        ),
        migrations.AlterField(
            model_name='rpmrepository',
            name='metadata_signing_service',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='core.asciiarmoreddetachedsigningservice'),
        ),
        migrations.AlterField(
            model_name='rpmrepository',
            name='repository_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.repository'),
        ),
        migrations.AlterField(
            model_name='ulnremote',
            name='remote_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.remote'),
        ),
        migrations.AlterField(
            model_name='updatecollection',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='updatecollectionpackage',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='updaterecord',
            name='content_ptr',
            field=models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='core.content'),
        ),
        migrations.AlterField(
            model_name='updatereference',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
        migrations.AlterField(
            model_name='variant',
            name='pulp_id',
            field=models.UUIDField(default=pulpcore.app.models.base.pulp_uuid, editable=False, primary_key=True, serialize=False),
        ),
    ]
