# Generated by Django 4.2.10 on 2024-04-25 16:39

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("rpm", "0061_fix_modulemd_defaults_digest"),
    ]

    operations = [
        migrations.CreateModel(
            name="RpmPackageSigningService",
            fields=[
                (
                    "signingservice_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="core.signingservice",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("core.signingservice",),
        ),
        migrations.AddField(
            model_name="rpmrepository",
            name="package_signing_fingerprint",
            field=models.TextField(max_length=40, null=True),
        ),
        migrations.AddField(
            model_name="rpmrepository",
            name="package_signing_service",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="rpm.rpmpackagesigningservice",
            ),
        ),
    ]
