# Generated by Django 2.2.14 on 2020-08-03 18:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0017_merge_advisory_collections'),
    ]

    operations = [
        migrations.AddField(
            model_name='updatecollection',
            name='_update_record',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='_collections', to='rpm.UpdateRecord'),
        ),
    ]
