# Generated by Django 4.2.4 on 2023-09-06 14:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("rpm", "0052_modulemd_digest"),
    ]

    operations = [
        migrations.AddField(
            model_name="rpmdistribution",
            name="generate_repo_config",
            field=models.BooleanField(default=False),
        ),
    ]
