# Generated by Django 4.2.17 on 2024-12-05 18:54

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0039_disttree_digest'),
    ]

    operations = [
        migrations.CreateModel(
            name='RpmAlternateContentSource',
            fields=[
                ('alternatecontentsource_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='rpm_rpmalternatecontentsource', serialize=False, to='core.alternatecontentsource')),
            ],
            options={
                'default_related_name': '%(app_label)s_%(model_name)s',
            },
            bases=('core.alternatecontentsource',),
        ),
    ]
