# Generated by Django 2.2.24 on 2021-08-03 21:04

from django.db import migrations


def migrate_sub_repo_value(apps, schema_editor):
    RpmRepository = apps.get_model('rpm', 'RpmRepository')
    RpmRepository.objects.filter(sub_repo=True).update(user_hidden=True)


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0036_checksum_type'),
    ]

    operations = [
        migrations.RunPython(
            migrate_sub_repo_value
        ),
        migrations.RemoveField(
            model_name='rpmrepository',
            name='sub_repo',
        ),
    ]
